# Documentation string. I wonder what ever refers to it? 

__doc__ = """IMAP Store product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB


import traceback
import string
import OFS.SimpleItem
import OFS.Folder
import Acquisition
import AccessControl.Role
import OFS.ObjectManager
from AccessControl.Permission import Permission

manage_addBaseStoreForm = HTMLFile('BaseStoreAdd', globals()) 

def manage_addBaseStore(self, id, baseUrl='', defaultTheme='', defaultStore='', REQUEST=None):
    """Add an BaseStore to a folder."""
    self._setObject(id, BaseStore(id, baseUrl, defaultTheme, defaultStore))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class BaseStore(
    OFS.ObjectManager.ObjectManager, 
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """IMAPStore object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """

    def __init__(self, id, baseUrl='', defaultTheme='', defaultStore=''): 
        """initialise a new instance of IMAPStore"""
        OFS.Folder.manage_addFolder(self.__of__(self), 'Themes', 'Themes')
        self.manage_edit(baseUrl, defaultTheme, defaultStore)
        self.id = id
        self.filterTerm = ''

        
    meta_type = 'BaseStore' # what do people think they're adding? 
    manage_properties = HTMLFile('BaseStoreEdit', globals()) # Management Interface
    
    __ac_permissions__=( # what permissions make sense for us? 
        ('View management screens', ('manage_tabs','manage_main','manage_workspace', 'manage_properties',
                                     'manage_edit')),
        ('Change permissions'     , ('manage_access',)           ),
        ('View'         , ('',)                        ),
        )

    manage_options=(
        {'label':'Contents',          'action':'manage_main'},
        {'label':'Properties',        'action':'manage_properties'},
        {'label':'Security',          'action':'manage_access'},
        )

    def getStore(self, key, REQUEST, dontUseCache=0):
        """
        """
        return getattr(self, key)
        if self.userCaches.has_key(REQUEST.AUTHENTICATED_USER.getUserName()):
            cache = self.userCaches[REQUEST.AUTHENTICATED_USER.getUserName()]
        else:
            cache = {}
        if key == '' : key = 'root'
        if not cache.has_key(key) or dontUseCache == 1:
            if key == 'NewMessage':
                cache[key] = None
                res = GenericMessage.GenericMessage(key, self, REQUEST)
            else:
                # try and select the folder...
                conn = REQUEST.AUTHENTICATED_USER.getConnection()
                if key == 'root': key = '/'
                err, result = conn.lsub('"%s"' % key, "%%")
                if len(result) == 1:
                    return None
                if key == '/': key = 'root'
                res = BaseFolder.BaseFolder(key, self, REQUEST)
            res = res.__of__(self)
            cache[key] = res
            get_transaction().commit()
        self.userCaches[REQUEST.AUTHENTICATED_USER.getUserName()] = cache
        get_transaction().commit()
        return cache[key]

    def getLongName(self, REQUEST):
        """
        """
        return ''


    def getParents(self, REQUEST):
        """
        """
        return [self]

    def getTheme(self, REQUEST):
        """
        """
        return getattr(self.Themes, self.defaultTheme)

    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))


    def setFilter(self, filterTerm, REQUEST):
        self.filterTerm = filterTerm

    def getURL(self, REQUEST):
        return "%s/%s" % (self.baseUrl, self.id)

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        return REQUEST['RESPONSE'].redirect("%s/%s?skin=index_html" % (self.getURL(REQUEST) , self.defaultStore))

        retVal = "<HTML><HEAD><TITLE>%s</TITLE></HEAD><BODY><H1>%s</H1><UL>" %  (self.getDescription(REQUEST),
                                                                                 self.getDescription(REQUEST))
        for store in filter(lambda i: not i is None, self.getStores(REQUEST)):
            retVal = '%s<LI><A HREF="%s">%s</A>' % (retVal, store.id, store.getDescription(REQUEST))
        retVal = '%s<HR>%s</BODY></HTML>' % (retVal, self.getData(REQUEST))
        return retVal
        return 'index_html NotImplemented'
        if 'skin' in REQUEST.form.keys():
            return getattr(self, REQUEST.form['skin']).render(self, REQUEST)
        else:
            return getattr(self, self.defaultSkin).render(self, REQUEST)

    manageProperties = HTMLFile('BaseStoreEdit', globals()) # Management Interface
    
    def manage_edit(self, baseUrl, defaultTheme, defaultStore,  REQUEST=None): 
        """does this really need a doc string?"""
        self.baseUrl = baseUrl
        self.defaultStore = defaultStore
        self.defaultTheme = defaultTheme
        if REQUEST is not None:
            return MessageDialog(
                title = 'Edited',
                message = "Properties for %s changed." % self.id,
                action = './manage_main',
                )

    def reset(self, REQUEST):
        """
        """
        self.userCaches = {}
        return 'RESET'

    def getDescription(self, REQUEST):
        """
        """
        return "IMAP Store @ %s" % self.acl_users.imap_server

    def subscribe(self, folderName, REQUEST):
        """
        Subscribe to a system
        """
        conn = REQUEST.AUTHENTICATED_USER.getConnection()
        err, result = conn.subscribe('"%s"' % folderName)
        if err == 'OK':
            return 1
        else:
            return 0
        

    def getStores(self, REQUEST):
        traceback.print_stack
        return map(lambda a, self=self, REQUEST=REQUEST: self.getStore(a, REQUEST),
                   filter(lambda a: not a in ('Themes', 'smtp', 'temp'), self.objectIds()))
        conn = REQUEST.AUTHENTICATED_USER.getConnection()
        err, result = conn.lsub('""', '"%"')
        #err, result2 = conn.lsub('"/"', '"%"')
        #result = result + result2
        if result == [None]:
            # We aren't subscribed to anything so lets go subscribe to at least the inbox...
            if self.subscribe('INBOX', REQUEST) == 1:
                # We subscribed fine so lets try and getStores again
                return self.getStores(REQUEST)
            else:
                raise "Couldn't subscribe to inbox - failing"
        retVal = []
        foldername = []
        for each in result:
            folder_name = string.join(string.split(each)[2:])
            if folder_name[0] == '"': folder_name = folder_name[1:-1]
            delim = string.split(each)[-2]
            if delim[0] == '"': delim = delim[1:-1]
            if folder_name == '': folder_name = 'root'
            if not string.upper(folder_name) in foldername:
                foldername.append(string.upper(folder_name))
                retVal.append(self.getStore(folder_name, REQUEST))
        return retVal

    def getData(self, REQUEST):
        """
        """
        return ''

    def getSearchTerm(self, REQUEST):
        """
        """
        return ''
