__doc__ = """IMAP Store product module."""

__version__ = '0.1'

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import time
import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import ImapMessage
import ImapFolder
import ImapAbstract
import ExtensionClass
from Products.IMAPAdapter import IMAPAdapter

class ImapFolder(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager, # Security manager.
    ExtensionClass
    ): 
        
    meta_type = 'ImapFolder' # what do people think they're adding? 

    def __init__(self, id, realParent, REQUEST): 
        """initialise a new instance of IMAPStore"""
        self.realParent = realParent
        self.id = id
        self.cacheUpdate = 1
        self.sortTerm = ''
        self.searchTerm = ''
        self.filterTerm = ''
        self.sortReverse = 0
        self.storedMessages = {}

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        if 'skin' in REQUEST.form.keys():
            REQUEST['store'] = self
            return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           REQUEST.form['skin'])(self.getParents(REQUEST)[-1].getTheme(REQUEST), REQUEST)
        else:

            retVal = "<HTML><HEAD><TITLE>%s</TITLE></HEAD><BODY><H1>%s</H1><UL>" %  (self.getDescription(REQUEST),
                                                                                     self.getDescription(REQUEST))
            for store in filter(lambda i: not i is None, self.getStores(REQUEST)):
                retVal = '%s<LI><A HREF="%s">%s</A>' % (retVal, store.id, store.getDescription(REQUEST))
            retVal = '%s<HR>%s</BODY></HTML>' % (retVal, self.getData(REQUEST))
            return retVal
    
    def skinaction(self, REQUEST):
        """ this needs a doc string"""

        return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           'skinaction')(self, REQUEST)

    def getHeader(self, headerName, REQUEST):
        """
        """
        return 'seld.id: %s : %s ' % (self.id, headerName)

    def getParents(self, REQUEST):
        """
        """
        retVal =  [self] + self.getParent(REQUEST).getParents(REQUEST)
        return retVal

    def isAbstract(self):
        return 0

    def getParent(self, REQUEST):
        return self.realParent
    
    def getURL(self, REQUEST):
        """
        """
        return '%s/%s' % (self.getParent(REQUEST).getURL(REQUEST), self.id)

    def getLongName(self, REQUEST):
        """
        """
        if self.id == 'root':
            return '/'
        return '%s%s/' % (self.getParent(REQUEST).getLongName(REQUEST), self.id)


    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))

    def getMIMEMajorType(self, REQUEST):
        """

        """
        (retVal, _) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal

    def getMIMEMinorType(self, REQUEST):
        """

        """
        (_, retVal) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal

    def getMIMEType(self, REQUEST):
        """
        Work out our mime type
        """
        return 'MULTIPART/X-DIRECTORY'

    def setup(self, REQUEST):
        """
        """
        self.REQUEST = REQUEST


    def getStores(self, REQUEST):
        """
        """
        retVal = []
        
        user = REQUEST.AUTHENTICATED_USER

        normalMessages = 1
        folderMessages = 1

        if self.filterTerm == 'not multipart/x-directory':
            folderMessages = 0
        if self.filterTerm == 'multipart/x-directory':
            normalMessages = 0

        if normalMessages == 1:
            def storeMessage(id, sm=self.storedMessages, obj=self, REQUEST=REQUEST):
                sm[id] = ImapMessage.ImapMessage(id, obj, REQUEST).__of__(obj)
            def checkMessage(id, sm=self.storedMessages):
                return not sm.has_key(id)
            uids = self.getUIDS(REQUEST)
            uids_to_get = filter(checkMessage, uids)
            map(storeMessage, uids_to_get)
            retVal = map(lambda n, sm=self.storedMessages: sm[n], uids)
        
        if folderMessages == 1:
            conn = REQUEST.AUTHENTICATED_USER.getConnection()
            folder_name =  self.getLongName(REQUEST)
            if folder_name[0] == '"': folder_name = folder_name[1:-1]
            err, result = conn.lsub('"%s"' % folder_name, '"%"')
            if result == [None]: result = []
            foldername = []
            for each in result:
                folder_name = string.split(each)[-1]
                delim = string.split(each)[-2]
                if delim[0] == '"': delim = delim[1:-1]
                folder_list = string.split(folder_name, delim)
                folder_name = folder_list[-1]
                
                if not folder_name in foldername and len(folder_list) > 1 and \
                   string.join(folder_list[:-1],delim) == self.getLongName(REQUEST)[:-1]:
                    foldername = foldername  + [folder_name]
                    retVal = retVal + [self.getStore(folder_name, REQUEST)]
        return retVal

    def getNumberOfParts(self, REQUEST):
        """
        """
        return len(self.getStores(REQUEST));

    def getStore(self, key, REQUEST):
        """
        """
        if not self.storedMessages.has_key(key):
            uids = map(lambda a:"%s" % a, self.getUIDS(REQUEST))
            if key in uids:
                mess = (ImapMessage.ImapMessage(key, self, REQUEST))
                mess = mess.__of__(self)
            else:
                conn = REQUEST.AUTHENTICATED_USER.getConnection()
                err, result = conn.lsub('"%s%s"' % (self.getLongName(REQUEST), key), "%%")
                if len(result) == 1:
                    return None

                mess = (ImapFolder(key, self, REQUEST))
                mess = mess.__of__(self)
            self.storedMessages[key]  = mess
        return self.storedMessages[key]

    def getID(self, REQUEST):
        return self.id

    def setFilter(self, filterTerm, REQUEST):
        self.filterTerm = filterTerm

    def getDescription(self, REQUEST):
        return "IMAP Folder: %s" % self.id

    def moveMessage(self, message, destination, REQUEST):
        """
        """
        if self.copyMessage(message, destination, REQUEST) == 1:
            self.deleteMessage(message, REQUEST)

    def copyMessage(self, message, destination, REQUEST):
        """
        """
        retVal = 1
        folderid = destination.getLongName(REQUEST)[:-1]
        messageid = message.id
        user = REQUEST['AUTHENTICATED_USER']
        conn = user.getConnection()
        typ, c = conn.select('"%s"' % self.getLongName(REQUEST)[:-1], 1)
        typ, c = conn.uid("COPY", messageid, folderid)
        destination.update(REQUEST)
        if typ == 'NO':
            retVal = 0
        return retVal

    def deleteMessage(self, message, REQUEST):
        """
        Description: Delete messages from the folder

        Parameters:
            messages: either a single 'Message' object or
               a single messageID or
               a list messageID's
            user: A user object. Must provide getConnection()

        Return: None
        """
        user = REQUEST.AUTHENTICATED_USER
        conn = user.getConnection()
        conn.select('"%s"' % self.getLongName(REQUEST)[:-1])
        typ, c = conn.uid("STORE", message.id, "+FLAGS", "(\Deleted)")
        typ, c = conn.expunge()
        self.cacheUpdate = 1 #Set cacheUpdate true since the folder contents have changed

    def update(self, REQUEST):
        """
        """
        self.cacheUpdate = 1
    
    def getUIDS(self, REQUEST):
        """
        Description: Return a list of UIDS for the folder.
                     This is affected by any term previously set
                     by the filter command

        Parameters:
            user: A user object. Must provide getConnection()

        Return: A list of UIDs
        
        Notes: This is an expensive function and we cache it.
        """
        if self.cacheUpdate == 1:
            self.setUIDS(REQUEST)
            self.cacheUpdate = 0
            return self.uids
        else:
            return self.uids

    def getSortTerm(self, REQUEST):
        return self.sortTerm

    def setSortTerm(self, sortTerm, REQUEST):
        self.sortTerm = sortTerm
        self.cacheUpdate = 1

    def getSearchTerm(self, REQUEST):
        return self.searchTerm

    def setSearchTerm(self, searchTerm, REQUEST):
        self.searchTerm = searchTerm
        self.cacheUpdate = 1

    def setReverse(self, REQUEST, setting=None):
        if setting != self.sortReverse:
            self.uids.reverse()
        if not setting is None:
            self.sortReverse = setting
        elif self.sortReverse == 1:
            self.sortReverse = 0
        else:
            self.sortReverse = 1


    def setUIDS(self, REQUEST):
        """
        Description: Update self.uids

        Parameters:
            user: A user object. Must provide getConnection()

        Notes: We don't want to display user preferences items so we retrieve
               messages without the content-type: text/x-notref headers

        TODO: get imap server capability and only use SORT when available
        """
        user = REQUEST.AUTHENTICATED_USER
        conn = user.getConnection()
        err, result = conn.select('"%s"' % self.getLongName(REQUEST)[:-1], 1)
        ids = []
        if err != 'NO':
            if self.sortTerm == '':
                err, result = conn.uid('search', None,
                                       '(text "%s" not header content-type "text/x-notpref")' %
                                       self.searchTerm)
            else:
                try:
                    err, result = conn.uid('sort', '(%s)' % self.sortTerm, 'utf-8',
                                           '(text "%s" not header content-type "text/x-notpref")' %
                                       self.searchTerm)
                except:
                    err, result = conn.uid('search', None,
                                           '(text "%s" not header content-type "text/x-notpref")' %
                                           self.searchTerm)

                    self.sortTerm = ''
            if result[0] is None:
                ids = []
            else:
                ids = (string.split(result[0]))
                # err, result = conn.logout()
                
            if self.sortReverse == 'true' or self.sortTerm == '':
                ids.reverse()
                ids = map(string.atoi, ids)
        self.uids = ids

    def getData(self, REQUEST):
        """
        """
        return ''

    def getAllSubscribed(self, REQUEST):
        """
        """
        return self.getParent(REQUEST).getAllSubscribed(REQUEST)

    def __bobo_traverse__(self, REQUEST, key):
        """

        """
        if hasattr(self, key):
            return getattr(self, key)
        else:
            result = ImapAbstract.ImapAbstract(key, self, REQUEST)
            result = result.__of__(self)
            return result
        
        #if key in self.getUIDS(REQUEST.AUTHENTICATED_USER):
        #    result =  ImapMessage.ImapMessage(key, REQUEST)
        #    result = result.__of__(self)
        #    return result
        try:
            result =  ImapMessage.ImapMessage(key, self, REQUEST)
            result = result.__of__(self)
            return result
        except:
            pass
        try:
            return getattr(self, key)
        except:
            raise 'Object: %s, not found' % key


    def isFirst(self, store, REQUEST):
        """
        """
        return "%s" % self.getUIDS(REQUEST)[0] == store.id

    def getNext(self, store, REQUEST):
        """
        """
        uids = self.getUIDS(REQUEST)
        try:
            for i in range(len(uids)):
                if "%s" % uids[i] == store.id:
                    return self.getStore(uids[i+1], REQUEST)
        except:
            raise 'There is no next UID'

    def getPrev(self, store, REQUEST):
        """
        """
        uids = self.getUIDS(REQUEST)
        try:
            for i  in range(len(uids)):
                if "%s" % uids[i] == store.id:
                    return self.getStore(uids[i-1], REQUEST)
        except:
            return 'There is no next UID'
            
    def isLast(self, store, REQUEST):
        """
        """
        return "%s" % self.getUIDS(REQUEST)[-1] == store.id
