__doc__ = """IMAP Message module."""

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import uu
import mimetools
import base64
import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import ImapMessage
import cStringIO
import ExtensionClass
from Products.IMAPAdapter import IMAPAdapter

class ImapMessage(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager, # Security manager.
    ExtensionClass
    ): 
        
    meta_type = 'ImapMessage' # what do people think they're adding? 


    def __init__(self, id, realParent, REQUEST, folder=None, isPart=0, partNumber='', headers=None): 
        """initialise a new instance of IMAPStore"""
        if type(id) == type(1): id = '%s' % id
        self.id = id
        if folder is None:
            self.cacheFolder = 1
        else:
            self.cacheFolder = 0 
            self.folder = folder
        self.isPart = isPart
        self.partNumber = partNumber
        self.realParent = realParent
        self.cacheStructure = 1
        self.cacheUpdate = 1
        self.cacheHeaders = 1
        self.textHeaders = headers
        self.sortTerm = ''
        self.searchTerm = ''
        self.sortReverse = 0

    def skinaction(self, REQUEST):
        """ this needs a doc string"""

        return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           'skinaction')(self, REQUEST)

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        if 'skin' in REQUEST.form.keys():
            REQUEST['store'] = self
            return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           REQUEST.form['skin'])(self.getParents(REQUEST)[-1].getTheme(REQUEST), REQUEST)
        else:
            if self.getData(REQUEST) != '' and self.getMIMEType != 'TEXT/PLAIN':
                REQUEST['RESPONSE'].setHeader('Content-type', self.getMIMEType(REQUEST))
                return self.getData(REQUEST)
            else:
                retVal = "<HTML><HEAD><TITLE>%s</TITLE></HEAD><BODY><H1>%s</H1><UL>" %  (self.getDescription(REQUEST),
                                                                                         self.getDescription(REQUEST))
                for store in self.getStores(REQUEST):
                    if store.getNumberOfParts(REQUEST) == 0:
                        retVal = '%s<LI><A HREF="%s">%s</A>' % (retVal,
                                                                store.getFileName(REQUEST),
                                                                store.getDescription(REQUEST))
                    else:
                        retVal = '%s<LI><A HREF="%s">%s</A>' % (retVal, store.id, store.getDescription(REQUEST))
                        retVal = '%s<HR>%s</BODY></HTML>' % (retVal, self.getData(REQUEST))
                return retVal

        if 'skin' in REQUEST.form.keys():
            return getattr(self, REQUEST.form['skin']).render(self, REQUEST)
        else:
            REQUEST['RESPONSE'].setHeader('Content-type', self.getMIMEType(REQUEST))
            return self.getData(REQUEST)

    def getLongName(self, REQUEST):
        """
        """
        pass

    def getData(self, REQUEST):
        """
        return a list of all messageID

        """
        if self.getNumberOfParts(REQUEST) == 0:

            user = REQUEST.AUTHENTICATED_USER
            conn = user.getConnection()
            blah =  self.getFolder(REQUEST)
            err, result = conn.select('"%s"' % self.getFolder(REQUEST).getLongName(REQUEST)[:-1], 1)
            if self.isPart == 1:
                err, result = conn.uid("fetch" ,  self.getUID(REQUEST), '(BODY[%s])' % self.partNumber[:-1])
            else:
                err, result = conn.uid("fetch" ,  self.getUID(REQUEST), '(BODY[%sTEXT])' % self.partNumber)
            try:
                (_, resultString) = result[0]
            except:
                resultString = ''
                # err, result = conn.logout()
        else:
            return '' #self.getMIMEType(REQUEST)
        # Decode data first !
        stuff = resultString
        encoding = string.strip(string.upper(string.split(self.getHeader('CONTENT-TRANSFER-ENCODING', REQUEST), ';')[0]))

        if encoding == "BASE64":		
            return base64.decodestring(stuff)
        elif encoding == "X-UUENCODE":
            stuff = filter(lambda l: ord(l) != 13, stuff)
            inStr = cStringIO.StringIO(stuff)
            outStr = cStringIO.StringIO()
            mimetools.decode(inStr, outStr, "uuencode")
            stuff = outStr.getvalue()
            inStr.close()
            outStr.close()
            return stuff
        elif encoding == "QUOTED-PRINTABLE":
            stuff = filter(lambda l: ord(l) != 13, stuff)
            inStr = cStringIO.StringIO(stuff)
            outStr = cStringIO.StringIO()
            mimetools.decode(inStr, outStr, "quoted-printable")
            stuff = outStr.getvalue()
            inStr.close()
            outStr.close()
            return stuff
        else:
            stuff = filter(lambda l: ord(l) != 13, stuff)
            return stuff

    def getNumberOfParts(self, REQUEST):
        """
        """
        if self.getMIMEMajorType(REQUEST) == 'MULTIPART':
            return len(self.getStructure(REQUEST)) - 4
        else:
            return 0
        
    def getStores(self, REQUEST):
        """
        """
        result = []
        if self.getMIMEType(REQUEST) == 'MESSAGE/RFC822':
            return self.getMessageRfc(REQUEST).getStores(REQUEST)
        for i in range(1, self.getNumberOfParts(REQUEST) + 1):
            result = result + [self.getMimePart(i, REQUEST)]
        return result

    def getID(self, REQUEST):
        return self.id

    def getDescription(self, REQUEST):
        return "IMAP Message: %s - %s" % (self.id, self.getHeader('subject', REQUEST))

    def getStructure(self, REQUEST):
        """
        """
        if self.cacheStructure == 1:
            self.setStructure(REQUEST)
            self.cacheStructure = 0
            return self.structure
        else:
            return self.structure

    def getHeaders(self, REQUEST):
        """
        Return dictionary of headers

        """
        if self.cacheHeaders == 1:
            self.setHeaders(REQUEST)
            self.cacheHeaders = 0
            return self.headers
        else:
            return self.headers

    def setHeaders(self, REQUEST):
        """
        Set headers
        """
        if self.textHeaders is None:
            user = REQUEST.AUTHENTICATED_USER
            conn = user.getConnection()
            err, result = conn.select('"%s"' % self.getFolder(REQUEST).getLongName(REQUEST)[:-1], 1)
            if self.isPart==2:
                err, result = conn.uid("fetch", self.getUID(REQUEST), '(BODY[%sHEADER])' % self.partNumber)
            elif self.isPart==1:
                err, result = conn.uid("fetch", self.getUID(REQUEST), '(BODY[%sMIME])' % self.partNumber)            
            else:
                err, result = conn.uid("fetch", self.getUID(REQUEST), '(BODY[HEADER])')

            try:
                (_, self.textHeaders) = result[0]
                self.headers = self.headers2dictionary(self.textHeaders)
            except:
                self.headers = {}
                
    def getUID(self, REQUEST):
        """
        """
        if self.isPart == 0:
            return self.id
        else:
            return self.getParent(REQUEST).getUID(REQUEST)

    def getFolder(self, REQUEST):
        """
        """
        if self.cacheFolder == 1:
            self.cacheFolder = 1
            if self.isPart == 0 or self.isPart == 2:
                self.folder =  self.getParent(REQUEST)
            else:
                blah = self.getParent(REQUEST)
                self.folder =  blah.getFolder(REQUEST)
        return self.folder
        

    def setFolder(self, REQUEST):
        """
        Set headers
        """
        self.folder = self.getParent(REQUEST)

    def getParents(self, REQUEST):
        """
        """
        retVal =  [self] + self.getParent(REQUEST).getParents(REQUEST)
        return retVal

    def getParent(self, REQUEST):
        return self.realParent.__of__(self.realParent.getParent(REQUEST))

    def isAbstract(self):
        return 0

    def getUIDS(self, REQUEST):
        """
        Description: Return a list of UIDS for the folder.
                     This is affected by any term previously set
                     by the filter command

        Parameters:
            user: A user object. Must provide getConnection()

        Return: A list of UIDs
        
        Notes: This is an expensive function and we cache it.
        """
        if self.cacheUpdate == 1:
            self.setUIDS(REQUEST)
            self.cacheUpdate = 0
            return self.uids
        else:
            return self.uids

    def setUIDS(self, REQUEST):
        """
        Description: Update self.uids

        Parameters:
            user: A user object. Must provide getConnection()

        Notes: We don't want to display user preferences items so we retrieve
               messages without the content-type: text/x-notref headers

        TODO: get imap server capability and only use SORT when available
        """
        user = REQUEST.AUTHENTICATED_USER
        conn = user.getConnection()
        err, result = conn.select('"%s"' % self.id, 1)

        if self.sortTerm == '':
            err, result = conn.uid('search', None,
                                   '(text "%s" not header content-type "text/x-notpref")' %
                                   self.searchTerm)
        else:
            try:
                err, result = conn.uid('sort', '(%s)' % self.sortTerm, 'utf-8',
                                       '(text "%s" not header content-type "text/x-notpref")' %
                                       self.searchTerm)
            except:
                err, result = conn.uid('search', None,
                                       '(text "%s" not header content-type "text/x-notpref")' %
                                       self.searchTerm)
                self.sortTerm = ''
        if result[0] is None:
            ids = []
        else:
            ids = (string.split(result[0]))
        # err, result = conn.logout()
        
        if self.sortReverse == 'true' or self.sortTerm == '':
            ids.reverse()
        ids = map(string.atoi, ids)
        self.uids = ids

    def getMimePart(self, part, REQUEST):
        """

        """
        result =  ImapMessage(part, self, REQUEST, self.getFolder(REQUEST), isPart=1, partNumber="%s%s." % (self.partNumber, part))
        if self.getMIMEType(REQUEST) == 'MULTIPART/DIGEST': # or result.getMIMEType(REQUEST) == 'MESSAGE/RFC822':
            result = result.getMessageRfc(REQUEST)
        result = result.__of__(self)

        return result
        if self.isPart == 1 or self.isPart == 2:
            return ImapMessage(part, self, self.getFolder(REQUEST), isPart=1, partNumber="%s.%s" % (self.partNumber, part))
        else:
            return ImapMessage(part, self, self.getFolder(REQUEST), isPart=1, partNumber=part)

    def setStructure(self, REQUEST):
        """

        """
        if self.isPart == 0:
            user = REQUEST.AUTHENTICATED_USER
            conn = user.getConnection()
            err, result = conn.select('"%s"' % self.getFolder(REQUEST).getLongName(REQUEST)[:-1], 1)
            err, result = conn.uid("fetch", self.id, '(BODYSTRUCTURE)')
            res2 = ''
            for x in result:
                res2 = res2 + string.join(x, '')
            result =  self.parseBodyStructure(res2)
            self.structure =  result[1][3]
        elif self.isPart == 1:
            self.structure =  self.getParent(REQUEST).getStructure(REQUEST)[string.atoi(self.id) - 1]
        else:
            self.structure =  self.getParent(REQUEST).getStructure(REQUEST)[string.atoi(self.id) - 1][8]
            
    def getStore(self, key, REQUEST):
        """
        TODO: Fix this
        """
        if key in ( map (lambda i: "%s" % i , range(1, self.getNumberOfParts(REQUEST)+1))):
            return self.getMimePart(key, REQUEST)
        else:
            stores = self.getStores(REQUEST)
            for each in stores:
                if each.getFileName(REQUEST) == key:
                    return each
        raise 'Message not found'

    def parseBodyStructure(self, text):
        """
        Parse the body strucutre return value
        """
        cur_str = ''
        result = []
        parenCount = 0
        inQuote = 'false'


        for ch in text:
            if ch == '(':
                parenCount = parenCount + 1
                if parenCount > 1:
                    cur_str = cur_str + ch
            elif ch == ')':
                parenCount =  parenCount - 1
                if parenCount == 0:
                    result.append(self.parseBodyStructure(cur_str))
                    cur_str = ''
                else:
                    cur_str = cur_str + ch
            elif parenCount == 0:
                if ch == '"':
                    if inQuote == 'true':
                        #result = result + [cur_str]
                        #cur_str = ''
                        inQuote = 'false'
                    else:
                        inQuote = 'true'
                elif ch in string.whitespace:
                    if inQuote != 'true':
                        if len(cur_str) > 0:
                            result.append(cur_str)
                            cur_str = ''
                    else:
                        cur_str = cur_str + ch
                else:
                    cur_str = cur_str + ch
            else:
                cur_str = cur_str + ch
        if len(cur_str) > 0:
            result.append(cur_str)
        dict = {}
        return result



    def getMIMEMajorType(self, REQUEST):
        """

        """
        (retVal, _) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal

    def getMIMEMinorType(self, REQUEST):
        """

        """
        (_, retVal) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal


    def getMIMEType(self, REQUEST):
        """
        Work out our mime type

        """
        #if self.isPart == 1:
        #    return self.mime
        mimeType = string.upper(string.split(self.getHeader('CONTENT-TYPE', REQUEST), ';')[0])
        if mimeType == '' : mimeType = 'TEXT/PLAIN'
        return string.split(mimeType, ';')[0]

    def getHeader(self, headerName, REQUEST):
        """
        return a list of all messageID

        """
        try:
            return string.strip(self.getHeaders(REQUEST)[string.upper(headerName)])
        except KeyError:
            return ''

        
    def headers2dictionary(self, text):
        """
        Taking text in the format of a header block convert to a dictionary
        """
        last_dict = None
        resultDictionary = {}
        if text != None:
            for each in string.split(text, '\r\n'):
                line = string.split(each, ':')
                if len(line) > 1:
                    last_dict = string.upper(line[0])
                    resultDictionary[string.upper(line[0])] = string.join(line[1:], ':')
                elif len(line[0]) > 1 and line[0][0] in string.whitespace and last_dict is not None:
                    resultDictionary[last_dict] = resultDictionary[last_dict] + line[0][1:]

        return resultDictionary

    def parseMimeStructure(self, text):
        """
        Parse the body strucutre return value
        """
        cur_str = ''
        result = []
        parenCount = 0
        inQuote = 'false'

        for ch in text:

            if ch == '(':
                parenCount = parenCount + 1
                if parenCount > 1:
                    cur_str = cur_str + ch
            elif ch == ')':
                parenCount =  parenCount - 1
                if parenCount == 0:
                    result.append(self.parseBodyStructure(cur_str))
                    cur_str = ''
                else:
                    cur_str = cur_str + ch
            elif parenCount == 0:
                if ch == '"':
                    if inQuote == 'true':
                        #result = result + [cur_str]
                        #cur_str = ''
                        inQuote = 'false'
                    else:
                        inQuote = 'true'
                elif ch in string.whitespace:
                    if inQuote != 'true':
                        if len(cur_str) > 0:
                            result.append(cur_str)
                            cur_str = ''
                    else:
                        cur_str = cur_str + ch
                else:
                    cur_str = cur_str + ch
            else:
                cur_str = cur_str + ch
        if len(cur_str) > 0:
            result.append(cur_str)
        dict = {}
        return result



    """

    def isFirst(self, REQUEST):

        return "%s" % self.getParent(REQUEST).getStores(REQUEST)[0].id == self.id

    def getNext(self, REQUEST):

        uids = self.getParent(REQUEST).getStores(REQUEST)
        try:
            for i  in range(len(uids)):
                if uids[i].id == self.id:
                    return uids[i+1]
        except:
            raise 'There is no next UID'

    def getPrev(self, REQUEST):

        uids = self.getFolder(REQUEST).getUIDS(REQUEST)
        try:
            for i  in range(len(uids)):
                if "%s" % uids[i] == self.id:
                    return uids[i-1]
        except:
            return 'There is no next UID'
            
    def isLast(self, REQUEST):

        return "%s" % self.getParent(REQUEST).getStores(REQUEST)[-1].id == self.id


    """






    def getFileName(self, REQUEST):
        """
        """
        # First try and get the filename attr of the CONTENT-DISPOSITION
        fileName = ''
        mimeType = (string.split(self.getHeader('CONTENT-DISPOSITION', REQUEST), ';'))
        for each in mimeType:
            parts = string.split(each, '=')
            if string.upper(string.strip(parts[0])) == 'FILENAME':
                fileName = string.strip(parts[1])
                break

        if fileName == '':
            fileName = '%s.ext' % self.id
        fileName = filter(lambda l: l != '"' and l not in string.whitespace, fileName)
        return fileName

    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))


    def getURL(self, REQUEST):
        """
        """
        if self.getNumberOfParts(REQUEST) == 0 and self.isPart != 0:
            return '%s/%s' % (self.getParent(REQUEST).getURL(REQUEST), self.getFileName(REQUEST))
        else:
            return '%s/%s' % (self.getParent(REQUEST).getURL(REQUEST), self.id)
        
    def getMessageRfc(self, REQUEST):
        """
        Return the RFC embedded part
        """
        return ImapMessage(self.id, self.getParent(REQUEST), REQUEST, self.getFolder(REQUEST), isPart=2, partNumber=self.partNumber)

