# Documentation string. I wonder what ever refers to it? 

__doc__ = """IMAP Store product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB


import traceback
import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import ImapFolder
import ImapAbstract
import GenericMessage
import OFS.ObjectManager
import ExtensionClass
from AccessControl.Permission import Permission
from Products.IMAPAdapter import IMAPAdapter

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addImapStoreForm = HTMLFile('ImapStoreAdd', globals()) 

def manage_addImapStore(self, id, imap_server='localhost', imap_port=143, REQUEST=None):
    """Add an ImapStore to a folder."""
    self._setObject(id, ImapStore(id, REQUEST.PARENTS[2], imap_server, imap_port))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class ImapStore(
    OFS.ObjectManager.ObjectManager, 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager, # Security manager.
    ExtensionClass
    ): 
    """IMAPStore object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """

    def __init__(self, id, realParent, imap_server='localhost', imap_port=143): 
        """initialise a new instance of IMAPStore"""
        IMAPAdapter.addIMAPAdapter(self.__of__(self), 'IMAPAdapter', imap_server, imap_port, '%sNotMail' % id)
        self.realParent = realParent
        self.id = id
        self.filterTerm = ''
        data=list(self.__ac_roles__)
        data.append('%sNotMail' % id)
        self.__ac_roles__=tuple(data)

        p=Permission('View ImapStore', ('',), self)
        p.setRoles(('%sNotMail' % id, ))

        self.userCaches = {}
        
    meta_type = 'ImapStore' # what do people think they're adding? 

    __ac_permissions__=( # what permissions make sense for us? 
        ('View management screens', ('manage_tabs','manage_main','manage_workspace')),
        ('Change permissions'     , ('manage_access',)           ),
        ('View ImapStore'         , ('', 'getURL')                        ),
        )

    manage_options=(
        {'label':'Contents',   'action':'manage_main'},
        {'label':'Security',   'action':'manage_access'},
        )

    def __bobo_traverse__(self, REQUEST, key):
        """

        """
        if hasattr(self, key):
            return getattr(self, key)
        else:
            result =  ImapAbstract.ImapAbstract(key, self, REQUEST)
            result = result.__of__(self)
            return result

    def getStore(self, key, REQUEST, dontUseCache=0):
        """
        """
        if self.userCaches.has_key(REQUEST.AUTHENTICATED_USER.getUserName()):
            cache = self.userCaches[REQUEST.AUTHENTICATED_USER.getUserName()]
        else:
            cache = {}
        if key == '' : key = 'root'
        if not cache.has_key(key) or dontUseCache == 1:
            if key == 'NewMessage':
                cache[key] = None
                res = GenericMessage.GenericMessage(key, self, REQUEST)
            else:
                # try and select the folder...
                conn = REQUEST.AUTHENTICATED_USER.getConnection()
                if key == 'root': key = '/'
                err, result = conn.lsub('"%s"' % key, "%%")
                if len(result) == 1:
                    return None
                if key == '/': key = 'root'
                res = ImapFolder.ImapFolder(key, self, REQUEST)
            res = res.__of__(self)
            cache[key] = res
            get_transaction().commit()
        self.userCaches[REQUEST.AUTHENTICATED_USER.getUserName()] = cache
        get_transaction().commit()
        return cache[key]

    def getLongName(self, REQUEST):
        """
        """
        return ''

    def getURL(self, REQUEST):
        """
        """
        return '%s/%s' % (self.getParent(REQUEST).getURL(REQUEST), self.id)

    def getParents(self, REQUEST):
        """
        """
        return [self] + self.getParent(REQUEST).getParents(REQUEST)

    def getNumberOfParts(self, REQUEST):
        """
        """
        return len(self.getStores(REQUEST))

    def skinaction(self, REQUEST):
        """ this needs a doc string"""

        return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           'skinaction')(self, REQUEST)


    def getParent(self, REQUEST):
        """
        """
        return self.realParent

    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))


    def setFilter(self, filterTerm, REQUEST):
        self.filterTerm = filterTerm

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""

        if 'skin' in REQUEST.form.keys():
            REQUEST['store'] = self
            return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           REQUEST.form['skin'])(self.getParents(REQUEST)[-1].getTheme(REQUEST), REQUEST)
        else:
            retVal = "<HTML><HEAD><TITLE>%s</TITLE></HEAD><BODY><H1>%s</H1><UL>" %  (self.getDescription(REQUEST),
                                                                                     self.getDescription(REQUEST))
            for store in filter(lambda i: not i is None, self.getStores(REQUEST)):
                retVal = '%s<LI><A HREF="%s">%s</A>' % (retVal, store.id, store.getDescription(REQUEST))
                retVal = '%s<HR>%s</BODY></HTML>' % (retVal, self.getData(REQUEST))
            return retVal


    manageProperties = HTMLFile('ImapStoreEdit', globals()) # Management Interface
    
    def manage_edit(self, defaultSkin, REQUEST=None): 
        """does this really need a doc string?"""
        self.defaultSkin = defaultSkin
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for %s changed." % self.id,
		action = './manage_main',
		)

    def reset(self, REQUEST):
        """
        """
        self.userCaches = {}
        return 'RESET'

    def getDescription(self, REQUEST):
        """
        """
        return "IMAP Store @ %s" % self.acl_users.imap_server

    def subscribe(self, folderName, REQUEST):
        """
        Subscribe to a system
        """
        conn = REQUEST.AUTHENTICATED_USER.getConnection()
        err, result = conn.subscribe('"%s"' % folderName)
        if err == 'OK':
            return 1
        else:
            return 0
        

    def getStores(self, REQUEST):
        conn = REQUEST.AUTHENTICATED_USER.getConnection()
        err, result = conn.lsub('""', '"%"')
        #err, result2 = conn.lsub('"/"', '"%"')
        #result = result + result2
        if result == [None]:
            # We aren't subscribed to anything so lets go subscribe to at least the inbox...
            if self.subscribe('INBOX', REQUEST) == 1:
                # We subscribed fine so lets try and getStores again
                return self.getStores(REQUEST)
            else:
                raise "Couldn't subscribe to inbox - failing"
        retVal = []
        foldername = []
        for each in result:
            folder_name = string.join(string.split(each)[2:])
            if folder_name[0] == '"': folder_name = folder_name[1:-1]
            delim = string.split(each)[-2]
            if delim[0] == '"': delim = delim[1:-1]
            if folder_name == '': folder_name = 'root'
            if not string.upper(folder_name) in foldername:
                foldername.append(string.upper(folder_name))
                retVal.append(self.getStore(folder_name, REQUEST))
        return retVal

    def getData(self, REQUEST):
        """
        """
        return ''

    def getSearchTerm(self, REQUEST):
        """
        """
        return ''
