# Documentation string. I wonder what ever refers to it? 

__doc__ = """IMAP Store product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import rfc822
from Products.IMAPAdapter import IMAPAdapter
from Products.MailHost.MailHost import SendMail

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addSmtpStoreForm = HTMLFile('SmtpStoreAdd', globals()) 

def manage_addSmtpStore(self, id, defaultDomain='', smtpServer='localhost', smtpPort=25, REQUEST=None):
    """Add an SmtpStore to a folder."""
    self._setObject(id, SmtpStore(id, defaultDomain, smtpServer, smtpPort))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your SmtpStore object."""

class SmtpStore(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """IMAPStore object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'SmtpStore' # what do people think they're adding? 
    manage_contents=OFS.Folder.Folder.manage_main

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change SmtpStore'     ,     ('manage_edit',)             ),
	('View SmtpStore',            ('',)                        ),
	)

    manage_options=(
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
	{'label': 'Security',   'action': 'manage_access'},
        )

    def __bobo_traverse__(self, REQUEST, key):
        """

        """
        if hasattr(self, key):
            return getattr(self, key)
        else:
            result =  SmtpAbstract.SmtpAbstract(key, self, REQUEST)
            result = result.__of__(self)
            return result

    def getStore(self, key, REQUEST):
        """
        """
        return None

    def getLongName(self, REQUEST):
        """
        """
        return ''

    def getURL(self, REQUEST):
        """
        """
        return '/'

    def getParents(self, REQUEST):
        """
        """
        return [self]
    

    def addStore(self, store, REQUEST):
        """
        """
        # Mail the store to give address..
        a = rfc822.AddrlistClass(store.getHeader('To', REQUEST) + ", " + store.getHeader('Cc', REQUEST))
        tolist = []
        for (name, addr) in a.getaddrlist(): tolist.append(addr)
        #tolist = map(parseaddr, filter(lambda s: s != '' , string.split(store.getHeader('To', REQUEST), ',') + string.split(store.getHeader('Cc', REQUEST), ',')))
        subj = store.getHeader('Subject', REQUEST)
        data = store.getAllData(REQUEST)
        mh = SendMail(self.smtpServer, self.smtpPort)
        mh.send('%s@%s' % (REQUEST.AUTHENTICATED_USER, self.defaultDomain),
                      tolist, subj, data)

    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))

    def __init__(self, id, defaultDomain='', smtpServer='localhost', smtpPort=25): 
        """initialise a new instance of IMAPStore"""
        self.id = id
        self.defaultDomain = defaultDomain
        self.smtpServer = smtpServer
        self.smtpPort = smtpPort

    def setFilter(self, filterTerm, REQUEST):
        self.filterTerm = filterTerm

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        if 'skin' in REQUEST.form.keys():
            return getattr(self, REQUEST.form['skin']).render(self, REQUEST)
        else:
            return getattr(self, self.defaultSkin).render(self, REQUEST)

    manage_main = HTMLFile('SmtpStoreEdit', globals()) # Management Interface
    
    def manage_edit(self, defaultSkin, REQUEST=None): 
        """does this really need a doc string?"""
        self.defaultSkin = defaultSkin

        if REQUEST is not None:
            return MessageDialog(
                title = 'Edited',
                message = "Properties for %s changed." % self.id,
                action = './manage_main',
            )

    def reset(self, REQUEST):
        """
        """
        self.userCaches = {}
        return 'RESET'

    def getDescription(self, REQUEST):
        """
        """
        return "I am an IMAP Store~!"

    def subscribe(self, folderName, REQUEST):
        """
        Subscribe to a system
        """
        conn = REQUEST.AUTHENTICATED_USER.getConnection()
        err, result = conn.subscribe('"%s"' % folderName)
        if err == 'OK':
            return 1
        else:
            return 0
        

    def getStores(self, REQUEST):
        return []

    def getData(self, REQUEST):
        """
        """
        return ''

    def getSearchTerm(self, REQUEST):
        """
        """
        return ''
