# Documentation string. I wonder what ever refers to it? 

__doc__ = """IMAP Store product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import uu
import mimetools
import base64
import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import cStringIO
import TextMessage
from Products.IMAPAdapter import IMAPAdapter

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

class GenericMessage(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """IMAPStore object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    def __init__(self, id, realParent, REQUEST, folder=None, isPart=0, partNumber=''): 
        """initialise a new instance of IMAPStore"""
        if type(id) == type(1): id = '%s' % id
        self.id = id
        self.headers = {}

        self.messages = {}
        self.messages['1'] = TextMessage.TextMessage('1', self, REQUEST, folder)
        if folder is None:
            self.cacheFolder = 1
        else:
            self.cacheFolder = 0 
        self.folder = folder
        self.isPart = isPart
        self.partNumber = partNumber
        self.realParent = realParent
        self.cacheStructure = 1
        self.cacheUpdate = 1
        self.cacheHeaders = 1
        self.mimeboundary = mimetools.choose_boundary()
        self.headers['CONTENT-TYPE'] = 'MULTIPART/MIXED; boundary = %s' % self.mimeboundary
        self.headers['MIME-VERSION'] = '1.0'
        self.sortTerm = ''
        self.searchTerm = ''
        self.sortReverse = 0

    def skinaction(self, REQUEST):
        """ this needs a doc string"""
        return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           'skinaction')(self, REQUEST)

    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        if 'skin' in REQUEST.form.keys():
            REQUEST['store'] = self
            return getattr(self.getParents(REQUEST)[-1].getTheme(REQUEST),
                           REQUEST.form['skin'])(self.getParents(REQUEST)[-1].getTheme(REQUEST), REQUEST)
        else:
            REQUEST['RESPONSE'].setHeader('Content-type', self.getMIMEType(REQUEST))
            return self.getData(REQUEST)


    manage_main = HTMLFile('ImapStoreEdit', globals()) # Management Interface

    def getLongName(self, REQUEST):
        """
        """
        pass

    def getAllData(self, REQUEST):
        """
        """
        body = self.getTextHeaders(REQUEST) + '\n\n\n' 
        for x in range(1, self.getNumberOfParts(REQUEST) + 1):
            body =  '%s\n\n--%s\n%s' %  (body, self.mimeboundary, self.getStore("%s" % x, REQUEST).getAllData(REQUEST))
        return body

    def getTextHeaders(self, REQUEST):
        """
        """
        retVal = ''
        for each in self.headers.keys():
            retVal = retVal + '%s: %s\n' % (string.capitalize(each), self.headers[each])
        return retVal

    def getData(self, REQUEST):
        """
        return a list of all messageID

        """
        if self.getNumberOfParts(REQUEST) == 0:

            user = REQUEST.AUTHENTICATED_USER
            conn = user.getConnection()
            blah =  self.getFolder(REQUEST)
            err, result = conn.select('"%s"' % self.getFolder(REQUEST).getLongName(REQUEST)[:-1], 1)
            if self.isPart == 1:
                err, result = conn.uid("fetch" ,  self.getUID(REQUEST), '(BODY[%s])' % self.partNumber[:-1])
            else:
                err, result = conn.uid("fetch" ,  self.getUID(REQUEST), '(BODY[%sTEXT])' % self.partNumber)
            try:
                (_, resultString) = result[0]
            except:
                resultString = ''
                # err, result = conn.logout()
        else:
            return self.getMIMEType(REQUEST)
        # Decode data first !
        stuff = resultString


        encoding = string.strip(string.upper(string.split(self.getHeader('CONTENT-TRANSFER-ENCODING', REQUEST), ';')[0]))
        


        if encoding == "BASE64":		
            return base64.decodestring(stuff)
        elif encoding == "X-UUENCODE":
            stuff = filter(lambda l: ord(l) != 13, stuff)
            inStr = cStringIO.StringIO(stuff)
            outStr = cStringIO.StringIO()
            mimetools.decode(inStr, outStr, "uuencode")
            stuff = outStr.getvalue()
            inStr.close()
            outStr.close()
            return stuff
        elif encoding == "QUOTED-PRINTABLE":
            stuff = filter(lambda l: ord(l) != 13, stuff)
            inStr = cStringIO.StringIO(stuff)
            outStr = cStringIO.StringIO()
            mimetools.decode(inStr, outStr, "quoted-printable")
            stuff = outStr.getvalue()
            inStr.close()
            outStr.close()
            return stuff
        else:
            stuff = filter(lambda l: ord(l) != 13, stuff)
            return stuff

    def getNumberOfParts(self, REQUEST):
        """
        """
        return len(self.messages)

    def addStore(self, store, REQUEST):
        """
        """
        # Code to add store..
        store.__of__(self)
        store.setParent(self, REQUEST)
        self.messages["%s" % (len(self.messages) + 1)] = store

    def clear(self, REQUEST):
        """
        """
        self.headers = {}
        self.messages = {}
        self.messages['1'] = TextMessage.TextMessage('1', self, REQUEST, self.folder)
        
    def getStores(self, REQUEST):
        """
        """
        return self.messages.values()

    def getID(self, REQUEST):
        return self.id

    def getDescription(self, REQUEST):
        
        return "Attachement: %s (%s)" % (self.getFileName(REQUEST), self.getMIMEType(REQUEST))

    def getHeaders(self, REQUEST):
        """
        Return dictionary of headers

        """
        return self.headers

    def getFolder(self, REQUEST):
        """
        """
        return self.folder

    def getParents(self, REQUEST):
        """
        """
        retVal =  [self] + self.getParent(REQUEST).getParents(REQUEST)
        return retVal

    def getParent(self, REQUEST):
        return self.realParent

    def getStore(self, key, REQUEST):
        """
        TODO: Fix this
        """
        return self.messages[key]

    def getMIMEMajorType(self, REQUEST):
        """

        """
        (retVal, _) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal

    def getMIMEMinorType(self, REQUEST):
        """

        """
        (_, retVal) = string.split(self.getMIMEType(REQUEST), '/')
        return retVal


    def getMIMEType(self, REQUEST):
        """
        Work out our mime type

        """
        #if self.isPart == 1:
        #    return self.mime
        mimeType = string.upper(string.split(self.getHeader('CONTENT-TYPE', REQUEST), ';')[0])
        if mimeType == '' : mimeType = 'TEXT/PLAIN'
        return string.split(mimeType, ';')[0]

    def getHeader(self, headerName, REQUEST):
        """
        return a list of all messageID

        """
        try:
            return string.strip(self.getHeaders(REQUEST)[string.upper(headerName)])
        except KeyError:
            return ''

    def setHeader(self, headerName, headerValue, REQUEST):
        self.getHeaders(REQUEST)[string.upper(headerName)] = headerValue
        
    def headers2dictionary(self, text):
        """
        Taking text in the format of a header block convert to a dictionary
        """
        last_dict = None
        resultDictionary = {}
        if text != None:
            for each in string.split(text, '\r\n'):
                line = string.split(each, ':')
                if len(line) > 1:
                    last_dict = string.upper(line[0])
                    resultDictionary[string.upper(line[0])] = string.join(line[1:], ':')
                elif len(line[0]) > 1 and line[0][0] in string.whitespace and last_dict is not None:
                    resultDictionary[last_dict] = resultDictionary[last_dict] + line[0][1:]

        return resultDictionary

    def parseMimeStructure(self, text):
        """
        Parse the body strucutre return value
        """
        cur_str = ''
        result = []
        parenCount = 0
        inQuote = 'false'

        for ch in text:

            if ch == '(':
                parenCount = parenCount + 1
                if parenCount > 1:
                    cur_str = cur_str + ch
            elif ch == ')':
                parenCount =  parenCount - 1
                if parenCount == 0:
                    result.append(self.parseBodyStructure(cur_str))
                    cur_str = ''
                else:
                    cur_str = cur_str + ch
            elif parenCount == 0:
                if ch == '"':
                    if inQuote == 'true':
                        #result = result + [cur_str]
                        #cur_str = ''
                        inQuote = 'false'
                    else:
                        inQuote = 'true'
                elif ch in string.whitespace:
                    if inQuote != 'true':
                        if len(cur_str) > 0:
                            result.append(cur_str)
                            cur_str = ''
                    else:
                        cur_str = cur_str + ch
                else:
                    cur_str = cur_str + ch
            else:
                cur_str = cur_str + ch
        if len(cur_str) > 0:
            result.append(cur_str)
        dict = {}
        return result



    """

    def isFirst(self, REQUEST):

        return "%s" % self.getParent(REQUEST).getStores(REQUEST)[0].id == self.id

    def getNext(self, REQUEST):

        uids = self.getParent(REQUEST).getStores(REQUEST)
        try:
            for i  in range(len(uids)):
                if uids[i].id == self.id:
                    return uids[i+1]
        except:
            raise 'There is no next UID'

    def getPrev(self, REQUEST):

        uids = self.getFolder(REQUEST).getUIDS(REQUEST)
        try:
            for i  in range(len(uids)):
                if "%s" % uids[i] == self.id:
                    return uids[i-1]
        except:
            return 'There is no next UID'
            
    def isLast(self, REQUEST):

        return "%s" % self.getParent(REQUEST).getStores(REQUEST)[-1].id == self.id


    """






    def getFileName(self, REQUEST):
        """
        """
        # First try and get the filename attr of the CONTENT-DISPOSITION
        fileName = ''
        mimeType = (string.split(self.getHeader('CONTENT-DISPOSITION', REQUEST), ';'))
        for each in mimeType:
            parts = string.split(each, '=')
            if string.upper(string.strip(parts[0])) == 'FILENAME':
                fileName = string.strip(parts[1])
                break

        if fileName == '':
            fileName = '%s.ext' % self.id
        fileName = filter(lambda l: l != '"' and l not in string.whitespace, fileName)
        return fileName

    def setClass(self, obj, REQUEST):
        """
        """
        obj.setRealObject(self.getStore(obj.id, REQUEST))

    def getURL(self, REQUEST):
        """
        """
        if self.isPart == 0:
            return ''
        else:
            return '%s%s/' % (self.getParent(REQUEST).getURL(REQUEST), self.id)

    def getMessageRfc(self, REQUEST):
        """
        Return the RFC embedded part
        """
        return ImapMessage(self.id, self.getParent(REQUEST), REQUEST, self.getFolder(REQUEST), isPart=2, partNumber=self.partNumber)

