# Documentation string. I wonder what ever refers to it? 

__doc__ = """IMAP Store product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

import string
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import ImapMessage
import ImapFolder
from Products.IMAPAdapter import IMAPAdapter
import traceback

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

class ImapAbstract(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """IMAPStore object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'ImapAbstract' # what do people think they're adding? 

    def __init__(self, id, realParent, REQUEST): 
        """initialise a new instance of IMAPStore"""
        self.id = id
        self.realObject = None
        self.haveRealObject = 0
        self.realParent = realParent

    def checkRealObject(self, REQUEST):
        """
        """
        if self.haveRealObject != 1:
            
            self.realParent.setClass(self, REQUEST)            


    def setRealObject(self, realObject):
        """
        """
        self.realObject = realObject
        self.haveRealObject = 1

    def isAbstract(self):
        return 1

    def __bobo_traverse__(self, REQUEST, key):
        """

        """
        if hasattr(self, key):
            return getattr(self, key)
        else:
            result = ImapAbstract(key, self, REQUEST)
            result = result.__of__(self)
            return result

    def getLongName(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        retVal = self.realObject.getLongName(REQUEST)
        return retVal

    def getParents(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getParents(REQUEST)

        
    def setFilter(self, filterString, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.setFilter(filterString, REQUEST)
        
    def setClass(self, obj, REQUEST=None):
        """ this needs a doc string"""
        self.checkRealObject(REQUEST)
        return self.realObject.setClass(obj, REQUEST)
    
    def index_html(self, REQUEST=None):
        """ this needs a doc string"""
        self.checkRealObject(REQUEST)
        if self.realObject is None:
            REQUEST['RESPONSE'].setStatus('NotFound')
            return REQUEST['RESPONSE'].notFoundError()
        return self.realObject.index_html(REQUEST)

    def skinaction(self, REQUEST=None):
        """ this needs a doc string"""
        self.checkRealObject(REQUEST)
        return self.realObject.skinaction(REQUEST)

    def getHeader(self, headerName, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getHeader(REQUEST)

    def getMIMEMajorType(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getMIMEMajorType(REQUEST)

    def getMIMEMinorType(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getMIMEMinorType(REQUEST)


    def getMIMEType(self, REQUEST):
        """
        Work out our mime type
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getMIMEType(REQUEST)


    def copyMessage(self, message, destination, REQUEST):
        """
        Work out our mime type
        """
        self.checkRealObject(REQUEST)
        return self.realObject.copyMessage(message, destination, REQUEST)

    def moveMessage(self, message, destination, REQUEST):
        """
        Work out our mime type
        """
        self.checkRealObject(REQUEST)
        return self.realObject.moveMessage(message, destination, REQUEST)

    def deleteMessage(self, message, REQUEST):
        """
        Work out our mime type
        """
        self.checkRealObject(REQUEST)
        return self.realObject.deleteMessage(message, REQUEST)

    def getStores(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getStores(REQUEST)

    def getURL(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getURL(REQUEST)
        

    def getStore(self, key, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getStore(key, REQUEST)


    def getNumberOfParts(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getNumberOfParts(REQUEST)

    def getID(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getID(REQUEST)

    def getDescription(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getDescription(REQUEST)

    def getData(self, REQUEST):
        """
        """
        self.checkRealObject(REQUEST)
        return self.realObject.getData(REQUEST)


    def getSortTerm(self, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.getSortTerm(REQUEST)

    def setSortTerm(self, sortTerm, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.setSortTerm(sortTerm, REQUEST)

    def getSearchTerm(self, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.getSearchTerm(REQUEST)

    def setSortTerm(self, sortTerm, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.setSearchTerm(sortTerm, REQUEST)

    def setReverse(self, REQUEST, setting=None):
        self.checkRealObject(REQUEST)
        return self.realObject.setReverse(REQUEST, setting)

    def isFirst(self, store, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.isFirst(store, REQUEST)

    def getNext(self, store, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.getNext(store, REQUEST)

    def getPrev(self, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.getPrev(store, REQUEST)
        
    def isLast(self, store, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.isLast(store, REQUEST)

    def update(self, REQUEST):
        self.checkRealObject(REQUEST)
        return self.realObject.update(REQUEST)
